instrument_prometheus (instrument v0.6.1)

Summary

Functions

Returns the Prometheus content type header value

Formats all registered metrics in Prometheus text exposition format

Functions

content_type()

-spec content_type() -> binary().

Returns the Prometheus content type header value

format()

-spec format() -> binary().

Formats all registered metrics in Prometheus text exposition format