instrument_prometheus (instrument v0.6.1)
View SourceSummary
Functions
Returns the Prometheus content type header value
Formats all registered metrics in Prometheus text exposition format
Returns the Prometheus content type header value
Formats all registered metrics in Prometheus text exposition format